@@ -14,10 +14,11 @@
 #    under the License.
 
 import json
+import re
 import six
 import time
 
-from six.moves.urllib import parse as urlparse
+from six.moves.urllib import parse
 from tempest import config
 from tempest.lib.common.utils import data_utils
 from tempest.lib import exceptions
@@ -54,25 +55,38 @@ class SharesV2Client(shares_client.SharesClient):
             new_headers = headers
         return new_headers
 
+    def verify_request_id(self, response):
+        response_headers = [r.lower() for r in response.keys()]
+        assert_msg = ("Response is missing request ID. Response "
+                      "headers are: %s") % response
+        assert 'x-compute-request-id' in response_headers, assert_msg
+
     # Overwrite all http verb calls to inject the micro version header
     def post(self, url, body, headers=None, extra_headers=False,
              version=LATEST_MICROVERSION):
         headers = self.inject_microversion_header(headers, version,
                                                   extra_headers=extra_headers)
-        return super(SharesV2Client, self).post(url, body, headers=headers)
+        resp, body = super(SharesV2Client, self).post(url, body,
+                                                      headers=headers)
+        self.verify_request_id(resp)
+        return resp, body
 
     def get(self, url, headers=None, extra_headers=False,
             version=LATEST_MICROVERSION):
         headers = self.inject_microversion_header(headers, version,
                                                   extra_headers=extra_headers)
-        return super(SharesV2Client, self).get(url, headers=headers)
+        resp, body = super(SharesV2Client, self).get(url, headers=headers)
+        self.verify_request_id(resp)
+        return resp, body
 
     def delete(self, url, headers=None, body=None, extra_headers=False,
                version=LATEST_MICROVERSION):
         headers = self.inject_microversion_header(headers, version,
                                                   extra_headers=extra_headers)
-        return super(SharesV2Client, self).delete(url, headers=headers,
-                                                  body=body)
+        resp, body = super(SharesV2Client, self).delete(url, headers=headers,
+                                                        body=body)
+        self.verify_request_id(resp)
+        return resp, body
 
     def patch(self, url, body, headers=None, extra_headers=False,
               version=LATEST_MICROVERSION):
@@ -84,19 +98,26 @@ class SharesV2Client(shares_client.SharesClient):
             version=LATEST_MICROVERSION):
         headers = self.inject_microversion_header(headers, version,
                                                   extra_headers=extra_headers)
-        return super(SharesV2Client, self).put(url, body, headers=headers)
+        resp, body = super(SharesV2Client, self).put(url, body,
+                                                     headers=headers)
+        self.verify_request_id(resp)
+        return resp, body
 
     def head(self, url, headers=None, extra_headers=False,
              version=LATEST_MICROVERSION):
         headers = self.inject_microversion_header(headers, version,
                                                   extra_headers=extra_headers)
-        return super(SharesV2Client, self).head(url, headers=headers)
+        resp, body = super(SharesV2Client, self).head(url, headers=headers)
+        self.verify_request_id(resp)
+        return resp, body
 
     def copy(self, url, headers=None, extra_headers=False,
              version=LATEST_MICROVERSION):
         headers = self.inject_microversion_header(headers, version,
                                                   extra_headers=extra_headers)
-        return super(SharesV2Client, self).copy(url, headers=headers)
+        resp, body = super(SharesV2Client, self).copy(url, headers=headers)
+        self.verify_request_id(resp)
+        return resp, body
 
     def reset_state(self, s_id, status="error", s_type="shares",
                     headers=None, version=LATEST_MICROVERSION,
@@ -139,6 +160,15 @@ class SharesV2Client(shares_client.SharesClient):
         self.expected_success(202, resp.status)
         return body
 
+    @staticmethod
+    def _get_base_url(endpoint):
+        url = parse.urlparse(endpoint)
+        # Get any valid path components before the version string
+        # regex matches version str & everything after (examples: v1, v2, v1.2)
+        base_path = re.split(r'(^|/)+v\d+(\.\d+)?', url.path)[0]
+        base_url = url._replace(path=base_path)
+        return parse.urlunparse(base_url) + '/'
+
     def send_microversion_request(self, version=None, script_name=None):
         """Prepare and send the HTTP GET Request to the base URL.
 
@@ -153,12 +183,19 @@ class SharesV2Client(shares_client.SharesClient):
         headers = self.get_headers()
         url, headers, body = self.auth_provider.auth_request(
             'GET', 'shares', headers, None, self.filters)
-        url = '/'.join(url.split('/')[:3]) + '/'
+        url = self._get_base_url(url)
         if script_name:
             url += script_name + '/'
         if version:
             headers[self.API_MICROVERSIONS_HEADER] = version
+
+        # Handle logging because raw_request doesn't log anything
+        start = time.time()
+        self._log_request_start('GET', url)
         resp, resp_body = self.raw_request(url, 'GET', headers=headers)
+        end = time.time()
+        self._log_request(
+            'GET', url, resp, secs=(end - start), resp_body=resp_body)
         self.response_checker('GET', resp, resp_body)
         resp_body = json.loads(resp_body)
         return resp, resp_body
@@ -243,7 +280,7 @@ class SharesV2Client(shares_client.SharesClient):
         """Get list of shares w/o filters."""
         headers = EXPERIMENTAL if experimental else None
         uri = 'shares/detail' if detailed else 'shares'
-        uri += '?%s' % urlparse.urlencode(params) if params else ''
+        uri += '?%s' % parse.urlencode(params) if params else ''
         resp, body = self.get(uri, headers=headers, extra_headers=experimental,
                               version=version)
         self.expected_success(200, resp.status)
@@ -284,7 +321,7 @@ class SharesV2Client(shares_client.SharesClient):
     def delete_share(self, share_id, params=None,
                      version=LATEST_MICROVERSION):
         uri = "shares/%s" % share_id
-        uri += '?%s' % (urlparse.urlencode(params) if params else '')
+        uri += '?%s' % (parse.urlencode(params) if params else '')
         resp, body = self.delete(uri, version=version)
         self.expected_success(202, resp.status)
         return body
@@ -300,7 +337,7 @@ class SharesV2Client(shares_client.SharesClient):
     def list_share_instances(self, version=LATEST_MICROVERSION,
                              params=None):
         uri = 'share_instances'
-        uri += '?%s' % urlparse.urlencode(params) if params else ''
+        uri += '?%s' % parse.urlencode(params) if params else ''
         resp, body = self.get(uri, version=version)
         self.expected_success(200, resp.status)
         return self._parse_resp(body)
@@ -419,7 +456,7 @@ class SharesV2Client(shares_client.SharesClient):
     def manage_share(self, service_host, protocol, export_path,
                      share_type_id, name=None, description=None,
                      is_public=False, version=LATEST_MICROVERSION,
-                     url=None):
+                     url=None, share_server_id=None):
         post_body = {
             "share": {
                 "export_path": export_path,
@@ -431,6 +468,8 @@ class SharesV2Client(shares_client.SharesClient):
                 "is_public": is_public,
             }
         }
+        if share_server_id is not None:
+            post_body['share']['share_server_id'] = share_server_id
         if url is None:
             if utils.is_microversion_gt(version, "2.6"):
                 url = 'shares/manage'
@@ -494,7 +533,7 @@ class SharesV2Client(shares_client.SharesClient):
                        version=LATEST_MICROVERSION):
         """Get list of share snapshots w/o filters."""
         uri = 'snapshots/detail' if detailed else 'snapshots'
-        uri += '?%s' % urlparse.urlencode(params) if params else ''
+        uri += '?%s' % parse.urlencode(params) if params else ''
         resp, body = self.get(uri, version=version)
         self.expected_success(200, resp.status)
         return self._parse_resp(body)
@@ -531,6 +570,8 @@ class SharesV2Client(shares_client.SharesClient):
             time.sleep(self.build_interval)
             body = self.get_snapshot(snapshot_id, version=version)
             snapshot_status = body['status']
+            if snapshot_status == status:
+                return
             if 'error' in snapshot_status:
                 raise (share_exceptions.
                        SnapshotBuildErrorException(snapshot_id=snapshot_id))
@@ -579,6 +620,12 @@ class SharesV2Client(shares_client.SharesClient):
         self.expected_success(202, resp.status)
         return body
 
+    def snapshot_reset_state(self, snapshot_id,
+                             status=constants.STATUS_AVAILABLE,
+                             version=LATEST_MICROVERSION):
+        self.reset_state(snapshot_id, status=status, s_type='snapshots',
+                         version=version)
+
 ###############
 
     def revert_to_snapshot(self, share_id, snapshot_id,
@@ -610,7 +657,7 @@ class SharesV2Client(shares_client.SharesClient):
                                    version=LATEST_MICROVERSION):
         uri = "types/%s/extra_specs" % share_type_id
         if params is not None:
-            uri += '?%s' % urlparse.urlencode(params)
+            uri += '?%s' % parse.urlencode(params)
         resp, body = self.get(uri, version=version)
         self.expected_success(200, resp.status)
         return self._parse_resp(body)
@@ -774,7 +821,7 @@ class SharesV2Client(shares_client.SharesClient):
         params = sorted(
             [(k, v) for (k, v) in list(metadata.items()) if v])
         if params:
-            query_string = "&%s" % urlparse.urlencode(params)
+            query_string = "&%s" % parse.urlencode(params)
 
         url = 'share-access-rules?share_id=%s' % share_id + query_string
         resp, body = self.get(url, version=version)
@@ -841,7 +888,7 @@ class SharesV2Client(shares_client.SharesClient):
             else:
                 url = 'os-services'
         if params:
-            url += '?%s' % urlparse.urlencode(params)
+            url += '?%s' % parse.urlencode(params)
         resp, body = self.get(url, version=version)
         self.expected_success(200, resp.status)
         return self._parse_resp(body)
@@ -854,7 +901,7 @@ class SharesV2Client(shares_client.SharesClient):
         if default:
             uri += '/default'
         if params is not None:
-            uri += '?%s' % urlparse.urlencode(params)
+            uri += '?%s' % parse.urlencode(params)
         resp, body = self.get(uri, version=version)
         self.expected_success(200, resp.status)
         return self._parse_resp(body)
@@ -1039,7 +1086,7 @@ class SharesV2Client(shares_client.SharesClient):
                           version=LATEST_MICROVERSION):
         """Get list of share groups w/o filters."""
         uri = 'share-groups%s' % ('/detail' if detailed else '')
-        uri += '?%s' % (urlparse.urlencode(params) if params else '')
+        uri += '?%s' % (parse.urlencode(params) if params else '')
         resp, body = self.get(uri, headers=EXPERIMENTAL, extra_headers=True,
                               version=version)
         self.expected_success(200, resp.status)
@@ -1135,7 +1182,7 @@ class SharesV2Client(shares_client.SharesClient):
                                version=LATEST_MICROVERSION):
         """Get list of share group types."""
         uri = 'share-group-types%s' % ('/detail' if detailed else '')
-        uri += '?%s' % (urlparse.urlencode(params) if params else '')
+        uri += '?%s' % (parse.urlencode(params) if params else '')
         resp, body = self.get(uri, headers=EXPERIMENTAL, extra_headers=True,
                               version=version)
         self.expected_success(200, resp.status)
@@ -1221,7 +1268,7 @@ class SharesV2Client(shares_client.SharesClient):
                                     version=LATEST_MICROVERSION):
         uri = "share-group-types/%s/group_specs" % share_group_type_id
         if params is not None:
-            uri += '?%s' % urlparse.urlencode(params)
+            uri += '?%s' % parse.urlencode(params)
         resp, body = self.get(uri, headers=EXPERIMENTAL, extra_headers=True,
                               version=version)
         self.expected_success(200, resp.status)
@@ -1285,7 +1332,7 @@ class SharesV2Client(shares_client.SharesClient):
                                    version=LATEST_MICROVERSION):
         """Get list of share group snapshots w/o filters."""
         uri = 'share-group-snapshots%s' % ('/detail' if detailed else '')
-        uri += '?%s' % (urlparse.urlencode(params) if params else '')
+        uri += '?%s' % (parse.urlencode(params) if params else '')
         resp, body = self.get(uri, headers=EXPERIMENTAL, extra_headers=True,
                               version=version)
         self.expected_success(200, resp.status)
@@ -1351,6 +1398,64 @@ class SharesV2Client(shares_client.SharesClient):
                            (sg_snapshot_name, status, self.build_timeout))
                 raise exceptions.TimeoutException(message)
 
+###############
+
+    def manage_share_server(self, host, share_network_id, identifier,
+                            driver_options=None, version=LATEST_MICROVERSION):
+        body = {
+            'share_server': {
+                'host': host,
+                'share_network_id': share_network_id,
+                'identifier': identifier,
+                'driver_options': driver_options if driver_options else {},
+            }
+        }
+
+        body = json.dumps(body)
+        resp, body = self.post('share-servers/manage', body,
+                               extra_headers=True, version=version)
+        self.expected_success(202, resp.status)
+        return self._parse_resp(body)
+
+    def unmanage_share_server(self, share_server_id,
+                              version=LATEST_MICROVERSION):
+        body = json.dumps({'unmanage': None})
+        resp, body = self.post('share-servers/%s/action' % share_server_id,
+                               body, extra_headers=True, version=version)
+        self.expected_success(202, resp.status)
+        return self._parse_resp(body)
+
+    def wait_for_share_server_status(self, server_id, status,
+                                     status_attr='status'):
+        """Waits for a share to reach a given status."""
+        body = self.show_share_server(server_id)
+        server_status = body[status_attr]
+        start = int(time.time())
+
+        while server_status != status:
+            time.sleep(self.build_interval)
+            body = self.show_share_server(server_id)
+            server_status = body[status_attr]
+            if server_status == status:
+                return
+            elif constants.STATUS_ERROR in server_status.lower():
+                raise share_exceptions.ShareServerBuildErrorException(
+                    server_id=server_id)
+
+            if int(time.time()) - start >= self.build_timeout:
+                message = ("Share server's %(status_attr)s failed to "
+                           "transition to %(status)s within the required "
+                           "time %(seconds)s." %
+                           {"status_attr": status_attr, "status": status,
+                            "seconds": self.build_timeout})
+                raise exceptions.TimeoutException(message)
+
+    def share_server_reset_state(self, share_server_id,
+                                 status=constants.SERVER_STATE_ACTIVE,
+                                 version=LATEST_MICROVERSION):
+        self.reset_state(share_server_id, status=status,
+                         s_type='share-servers', version=version)
+
 ###############
 
     def migrate_share(self, share_id, host,
@@ -1525,6 +1630,26 @@ class SharesV2Client(shares_client.SharesClient):
         self.expected_success(expected_status, resp.status)
         return self._parse_resp(body)
 
+    def list_share_replica_export_locations(self, replica_id,
+                                            expected_status=200,
+                                            version=LATEST_MICROVERSION):
+        uri = "share-replicas/%s/export-locations" % replica_id
+        resp, body = self.get(uri, headers=EXPERIMENTAL,
+                              extra_headers=True, version=version)
+        self.expected_success(expected_status, resp.status)
+        return self._parse_resp(body)
+
+    def get_share_replica_export_location(self, replica_id,
+                                          export_location_id,
+                                          expected_status=200,
+                                          version=LATEST_MICROVERSION):
+        uri = "share-replicas/%s/export-locations/%s" % (replica_id,
+                                                         export_location_id)
+        resp, body = self.get(uri, headers=EXPERIMENTAL,
+                              extra_headers=True, version=version)
+        self.expected_success(expected_status, resp.status)
+        return self._parse_resp(body)
+
     def wait_for_share_replica_status(self, replica_id, expected_status,
                                       status_attr='status'):
         """Waits for a replica's status_attr to reach a given status."""
@@ -1627,7 +1752,7 @@ class SharesV2Client(shares_client.SharesClient):
                             version=LATEST_MICROVERSION):
         """Get list of share networks w/o filters."""
         uri = 'share-networks/detail' if detailed else 'share-networks'
-        uri += '?%s' % urlparse.urlencode(params) if params else ''
+        uri += '?%s' % parse.urlencode(params) if params else ''
         resp, body = self.get(uri, version=version)
         self.expected_success(200, resp.status)
         return self._parse_resp(body)
@@ -1663,7 +1788,7 @@ class SharesV2Client(shares_client.SharesClient):
         resp, body = self.get("snapshots/%s/access-list" % snapshot_id,
                               version=LATEST_MICROVERSION)
         body = self._parse_resp(body)
-        found_rules = filter(lambda x: x['id'] == rule_id, body)
+        found_rules = [r for r in body if r['id'] == rule_id]
 
         return found_rules[0] if len(found_rules) > 0 else None
 
@@ -1755,7 +1880,7 @@ class SharesV2Client(shares_client.SharesClient):
     def list_messages(self, params=None, version=LATEST_MICROVERSION):
         """List all messages."""
         url = 'messages'
-        url += '?%s' % urlparse.urlencode(params) if params else ''
+        url += '?%s' % parse.urlencode(params) if params else ''
         resp, body = self.get(url, version=version)
         self.expected_success(200, resp.status)
         return self._parse_resp(body)
@@ -1830,7 +1955,7 @@ class SharesV2Client(shares_client.SharesClient):
         if detailed:
             uri += '/detail'
         if params:
-            uri += "?%s" % urlparse.urlencode(params)
+            uri += "?%s" % parse.urlencode(params)
         resp, body = self.get(uri, version=version)
         self.expected_success(200, resp.status)
         return self._parse_resp(body)
