@@ -687,18 +687,56 @@ class BaseSharesTest(test.BaseTestCase):
         return sg_snapshot
 
     @classmethod
-    def get_availability_zones(cls, client=None):
+    def get_availability_zones(cls, client=None, backends=None):
         """List the availability zones for "manila-share" services
 
          that are currently in "up" state.
          """
-        client = client or cls.shares_v2_client
+        client = client or cls.admin_shares_v2_client
+        backends = (
+            '|'.join(['^%s$' % backend for backend in backends])
+            if backends else '.*'
+        )
         cls.services = client.list_services()
         zones = [service['zone'] for service in cls.services if
-                 service['binary'] == "manila-share" and
-                 service['state'] == 'up']
+                 service['binary'] == 'manila-share' and
+                 service['state'] == 'up' and
+                 re.search(backends, service['host'])]
         return zones
 
+    @classmethod
+    def get_pools_matching_share_type(cls, share_type, client=None):
+        client = client or cls.admin_shares_v2_client
+        if utils.is_microversion_supported('2.23'):
+            return client.list_pools(
+                search_opts={'share_type': share_type['id']})['pools']
+
+        pools = client.list_pools(detail=True)['pools']
+        share_type = client.get_share_type(share_type['id'])['share_type']
+        extra_specs = {}
+        for k, v in share_type['extra_specs'].items():
+            extra_specs[k] = (
+                True if six.text_type(v).lower() == 'true'
+                else False if six.text_type(v).lower() == 'false' else v
+            )
+        return [
+            pool for pool in pools if all(y in pool['capabilities'].items()
+                                          for y in extra_specs.items())
+        ]
+
+    @classmethod
+    def get_availability_zones_matching_share_type(cls, share_type,
+                                                   client=None):
+
+        client = client or cls.admin_shares_v2_client
+        pools_matching_share_type = cls.get_pools_matching_share_type(
+            share_type, client=client)
+        backends_matching_share_type = set(
+            [pool['name'].split("#")[0] for pool in pools_matching_share_type]
+        )
+        azs = cls.get_availability_zones(backends=backends_matching_share_type)
+        return azs
+
     def get_pools_for_replication_domain(self):
         # Get the list of pools for the replication domain
         pools = self.admin_client.list_pools(detail=True)['pools']
@@ -1077,6 +1115,88 @@ class BaseSharesAdminTest(BaseSharesTest):
             name=share_group_type_name, share_types=[cls.share_type_id],
             client=cls.admin_shares_v2_client)
 
+    def _create_share_for_manage(self):
+        creation_data = {
+            'share_type_id': self.st['share_type']['id'],
+            'share_protocol': self.protocol,
+        }
+
+        share = self.create_share(**creation_data)
+        share = self.shares_v2_client.get_share(share['id'])
+
+        if utils.is_microversion_ge(CONF.share.max_api_microversion, "2.9"):
+            el = self.shares_v2_client.list_share_export_locations(share["id"])
+            share["export_locations"] = el
+
+        return share
+
+    def _unmanage_share_and_wait(self, share):
+        self.shares_v2_client.unmanage_share(share['id'])
+        self.shares_v2_client.wait_for_resource_deletion(share_id=share['id'])
+
+    def _reset_state_and_delete_share(self, share):
+        self.shares_v2_client.reset_state(share['id'])
+        self._delete_share_and_wait(share)
+
+    def _delete_snapshot_and_wait(self, snap):
+        self.shares_v2_client.delete_snapshot(snap['id'])
+        self.shares_v2_client.wait_for_resource_deletion(
+            snapshot_id=snap['id']
+        )
+        self.assertRaises(exceptions.NotFound,
+                          self.shares_v2_client.get_snapshot,
+                          snap['id'])
+
+    def _delete_share_and_wait(self, share):
+        self.shares_v2_client.delete_share(share['id'])
+        self.shares_v2_client.wait_for_resource_deletion(share_id=share['id'])
+        self.assertRaises(exceptions.NotFound,
+                          self.shares_v2_client.get_share,
+                          share['id'])
+
+    def _manage_share(self, share, name, description, share_server_id):
+        managed_share = self.shares_v2_client.manage_share(
+            service_host=share['host'],
+            export_path=share['export_locations'][0],
+            protocol=share['share_proto'],
+            share_type_id=self.share_type['share_type']['id'],
+            name=name,
+            description=description,
+            share_server_id=share_server_id
+        )
+        self.shares_v2_client.wait_for_share_status(
+            managed_share['id'], constants.STATUS_AVAILABLE
+        )
+
+        return managed_share
+
+    def _unmanage_share_server_and_wait(self, server):
+        self.shares_v2_client.unmanage_share_server(server['id'])
+        self.shares_v2_client.wait_for_resource_deletion(
+            server_id=server['id']
+        )
+
+    def _manage_share_server(self, share_server, fields=None):
+        params = fields or {}
+        managed_share_server = self.shares_v2_client.manage_share_server(
+            params.get('host', share_server['host']),
+            params.get('share_network_id', share_server['share_network_id']),
+            params.get('identifier', share_server['identifier']),
+        )
+        self.shares_v2_client.wait_for_share_server_status(
+            managed_share_server['id'],
+            constants.SERVER_STATE_ACTIVE,
+        )
+
+        return managed_share_server
+
+    def _delete_share_server_and_wait(self, share_server_id):
+        self.shares_v2_client.delete_share_server(
+            share_server_id
+        )
+        self.shares_v2_client.wait_for_resource_deletion(
+            server_id=share_server_id)
+
 
 class BaseSharesMixedTest(BaseSharesTest):
     """Base test case class for all Shares API tests with all user roles."""
